import { Component } from './index';
/**
 * Returns an element that will be replaced by the result of the given
 * promise when it resolves.
 */
export declare function async<E extends Element>(component: Promise<E>): E;
/**
 * Wraps a component that asynchronously resolves as a regular component
 * whose element will be replaced once the promise resolves.
 */
export declare function async<P extends {}, E extends Element>(component: (props: P) => Promise<E>): Component<P, E>;
/**
 * Given a component, some of its properties, and a promise that resolves
 * to the rest of its properties, returns an element that will be replaced
 * by the resolved element when the promise finishes.
 */
export declare function Async<P extends {}, E extends Element, K extends {}>({ component, props }: P & {
    component: Component<P & K, E>;
    props: Promise<K>;
}): E;
/**
 * Given a promise that resolves to a component and its properties, returns
 * an element that will be replaced by the resolved element when the promise finishes.
 */
export declare function Async<P extends {}, E extends Element>({ component, ...props }: P & {
    component: Promise<Component<P, E>>;
}): E;
