import * as Rx from 'rxjs';
/**
 * Defines an observable stream that notifies subscribers of a change of a value.
 */
export declare class Observable<T> implements Rx.Subscribable<T> {
    private val;
    private readonly observers;
    constructor(value: T);
    /**
     * Sets the underlying value without notifying subscribers of the change.
     */
    setUnderlyingValue(value: T): void;
    /** Gets or sets the underlying value.
     *
     * - When getting the value, only the last version is returned.
     * - When setting the value, also notifies all subscribers of the change.
     */
    value: T;
    /**
     * Returns the string representation of the underlying value.
     */
    toString(): string;
    /**
     * Subcribes to this reactive value.
     */
    subscribe(next?: Rx.PartialObserver<T> | ((value: T) => void), error?: (error: any) => void, complete?: () => void): Rx.Unsubscribable;
    /**
     * @see subscribe
     */
    observe(...args: any[]): Rx.Unsubscribable;
}
/**
 * {T} if {T} is {Observable}, and {Observable<T>} otherwise.
 */
export declare type Obs<T> = T extends Observable<infer _> ? T : Observable<T>;
/**
 * {T} or {Observable<T>}.
 */
export declare type ObservableLike<T> = T | Observable<T>;
/**
 * A component function.
 */
export declare type Component<P = {}, E extends Node = Element> = (props: P) => E;
type Slot = {
    elements: Node[];
    nextMarker: Node;
    hasDefault: boolean;
};
/**
 * Creates an {HTMLElement}, given its parent, tag, and attributes.
 *
 * @param parent The parent of the node to create, to which the node
 *   will be appended. Can be `null`.
 * @param tag The HTML tag of the element to create.
 * @param attrs An optional object that contains attributes that
 *   will be set on the created element.
 *
 * @returns The corresponding HTML element.
 */
export declare function createElement<K extends keyof HTMLElementTagNameMap>(tag: K, attrs: Partial<HTMLElementTagNameMap[K]>): HTMLElementTagNameMap[K];
/**
 * Creates an {HTMLElement}, given its parent, component, and attributes.
 *
 * @param parent The parent of the node to create, to which the node
 *   will be appended. Can be `null`.
 * @param tag The component of the element to create.
 * @param attrs An optional object that contains attributes and properties that
 *   will be set on the created element.
 *
 * @returns The corresponding HTML element.
 */
export declare function createElement<K extends (props: object) => Element>(component: K, attrs: K extends (props: infer A) => infer E ? A & Partial<E> : never): ReturnType<K>;
/**
 * Inserts an arbitrarily nested list of elements as a child of `parent`,
 * populating `inserted` at the same time with all the inserted elements.
 *
 * Not intended for direct use.
 */
export declare function addElement(parent: HTMLElement, elt: any, inserted: Node[], nextMarker: Node): any;
/**
 * Creates a marker node, which is an invisible node before which other elements can be inserted.
 *
 * Not intended for direct use.
 */
export declare function createMarker(): Node;
/**
 * Defines a new slot for an element.
 *
 * Not intended for direct use.
 */
export declare function defineSlot(element: JSX.Element, slotName: string, def?: Node[]): void;
/**
 * Destroys the given element, unsubscribing to all of its `subscriptions`.
 *
 * Not intended for direct use.
 */
export declare function destroy(this: JSX.Element): void;
/**
 * Appends the given element or group of elements to the
 * given slot.
 *
 * Not intended for direct use.
 */
export declare function appendToSlot(this: JSX.Element, slot: string, elt: any): void;
/**
 * Returns whether the given value is an `Observable` stream.
 */
export declare function isObservable<T>(value: ObservableLike<T>): value is Observable<T>;
/**
 * Returns an `Observable` stream that wraps the given value.
 *
 * If the given value is already an `Observable` stream, it is returned.
 */
export declare function observable<T>(value: ObservableLike<T>): Obs<T>;
/**
 * Returns the underlying value of the given observable.
 *
 * If the given observable is, in fact, not an observable, it is directly returned.
 */
export declare function value<T>(value: ObservableLike<T>): T extends Observable<infer V> ? V : T;
/**
 * Returns a computed value that is updated every time of one of the given
 * dependencies changes.
 */
export declare function computed<T>(dependencies: Observable<any>[], computation: () => T): Observable<T>;
/**
 * Merges multiple observable sequences together.
 */
export declare function merge(...observables: Rx.Subscribable<any>[]): Rx.Subscribable<any>;
declare global {
    namespace JSX {
        type Element = HTMLElement & {
            ondestroy?: () => void;
            readonly destroy: () => void;
            readonly appendToSlot: (slot: string, child: any) => void;
            readonly slots?: Record<string, Slot>;
            readonly subscriptions?: Rx.Unsubscribable[];
        };
        type IntrinsicElements = {
            [key in keyof HTMLElementTagNameMap]: Partial<HTMLElementTagNameMap[key]> & {
                class?: string;
                slot?: string;
                ref?: HTMLElementTagNameMap[key];
            };
        };
    }
}
export {};
